#! /usr/local/bin/ruby
# encoding: utf-8
#
# test_pg_exec.rb
# 函数の中でSQL文を実行するテスト。
#

require_relative 'am_assert';
include AmAssert;
require_relative 'pgx';
require		'bigdecimal';
require		'date';

# ======================================================================
#
class TestPgExec
	# ==============================================================
	#
	def initialize()
		@pq = Pq.new('postgresql://localhost/db_plrb');
	end

	# ==============================================================
	# 函数の中でPg::execute()を実行してテーブルを操作。
	#
	def test_pg_execute()

		@pq.exec("DELETE FROM music_tbl;");

		test_cases = [
			[ 'Violin', true, 2.4, 873 ],
			[ 'Timpani', false, 3.4, nil ],
			[ 'Flute', nil, 9.4, 123 ],
			[ 'Harp', true, 19, 1034 ],
		];
		sql = "SELECT insert_music($1, $2, $3, $4) AS result;";
		test_cases.each do |name, is_string, length, weight|
			rc = @pq.select_one(sql, name, is_string, length, weight);
		end

		sql = "SELECT * FROM music_tbl ORDER BY id;";
		result = @pq.select_all(sql);
		test_cases.each do |name, is_string, length, weight|
			record = result.shift;
			am_assert_equals(name, record['name']);
			if is_string != nil
				am_assert_equals(is_string, record['is_string']);
			else
				am_assert_equals(nil, record['is_string']);
			end
			if length != nil
				am_assert_equals(length, record['length']);
			else
				am_assert_equals(nil, record['length']);
			end
			if weight != nil
				am_assert_equals(weight, record['weight']);
			else
				am_assert_equals(nil, record['weight']);
			end
		end
	end

	# ==============================================================
	# 函数の中でPg::execute_with_type()を実行してテーブルを操作。
	#
	def test_pg_execute_with_type()

		@pq.exec("DELETE FROM music_tbl;");

		test_cases = [
			[ 'Violin', true, 2.4, 873 ],
			[ 'Timpani', false, 3.4, nil ],
			[ 'Flute', nil, 9.4, 123 ],
			[ 'Harp', true, 19, 1034 ],
		];
		sql = "SELECT insert_music_with_type($1, $2, $3, $4) AS result;";
		test_cases.each do |name, is_string, length, weight|
			rc = @pq.select_one(sql, name, is_string, length, weight);
		end

		sql = "SELECT * FROM music_tbl ORDER BY id;";
		result = @pq.select_all(sql);
		test_cases.each do |name, is_string, length, weight|
			record = result.shift;
			am_assert_equals(name, record['name']);
			if is_string != nil
				am_assert_equals(is_string, record['is_string']);
			else
				am_assert_equals(nil, record['is_string']);
			end
			if length != nil
				am_assert_equals(length, record['length']);
			else
				am_assert_equals(nil, record['length']);
			end
			if weight != nil
				am_assert_equals(weight, record['weight']);
			else
				am_assert_equals(nil, record['weight']);
			end
		end
	end

	# ==============================================================
	# 函数の中でPg::exec_prepared()を実行してテーブルを操作。
	#
	def test_pg_exec_prepared()

		@pq.exec("DELETE FROM music_tbl;");

		test_cases = [
			[ 'Violin', true, 2.4, 873 ],
			[ 'Timpani', false, 3.4, nil ],
			[ 'Flute', nil, 9.4, 123 ],
			[ 'Harp', true, 19, 1034 ],
		];
		sql = "SELECT insert_by_prepared($1, $2, $3, $4) AS result;";
		test_cases.each do |name, is_string, length, weight|
			rc = @pq.select_one(sql, name, is_string, length, weight);
		end

		sql = "SELECT * FROM music_tbl ORDER BY id;";
		result = @pq.select_all(sql);
		test_cases.each do |name, is_string, length, weight|
			record = result.shift;
			am_assert_equals(name, record['name']);
			if is_string != nil
				am_assert_equals(is_string, record['is_string']);
			else
				am_assert_equals(nil, record['is_string']);
			end
			if length != nil
				am_assert_equals(length, record['length']);
			else
				am_assert_equals(nil, record['length']);
			end
			if weight != nil
				am_assert_equals(weight, record['weight']);
			else
				am_assert_equals(nil, record['weight']);
			end
		end
	end

	# ==============================================================
	# 函数の中でPg::fetch()を実行してテーブルを操作。
	#
	def test_pg_fetch()

		@pq.exec("DELETE FROM music_tbl;");

		test_cases = [
			[ 'Violin', true, 2.4, 873 ],
			[ 'Timpani', false, 3.4, nil ],
			[ 'Flute', nil, 9.4, 123 ],
			[ 'Harp', true, 19, 1034 ],
		];
		sql = "INSERT INTO music_tbl (name, is_string, length, weight) VALUES ($1, $2, $3, $4);";
		test_cases.each do |name, is_string, length, weight|
			rc = @pq.execute(sql, name, is_string, length, weight);
		end

		guess_result = [
			{ "dt" => "(Flute,123)" },
			{ "dt" => "(Harp,1034)" },
			{ "dt" => "(Timpani,)" },
			{ "dt" => "(Violin,873)" },
		];
		sql = "SELECT fetch_as_set() AS dt;";
		result = @pq.execute(sql);

	end

	# ==============================================================
	# Pg::execute() に渡すプレースホルダーのさまざまな型。
	#
	def test_pg_execute_various_types()
		@pq.exec("DELETE FROM various_types_tbl;");

		test_cases = [
			[ 'v_boolean', true, false ],
			[ 'v_boolean', 't', false ],
			[ 'v_boolean', 1, false ],
			[ 'v_boolean', 10, true ],
			[ 'v_bigint', 0, false ],
			[ 'v_bigint', 9223372036854775807, false ],	# MAX
			[ 'v_bigint', -9223372036854775808, false ],	# MIN
			[ 'v_bigint', 9223372036854775808, true ],	# 範囲外
			[ 'v_integer', 0, false ],
			[ 'v_integer', 2147483647, false ],	# MAX
			[ 'v_integer', -2147483648, false ],	# MIN
			[ 'v_integer', 2147483648, true ],	# 範囲外
			[ 'v_integer', 24.34, true ],		# Float
			[ 'v_integer', '2147483647', false ],	# String
			[ 'v_integer', '24.34', true ],	# String
			[ 'v_integer', BigDecimal.new('2147483647'), false ],

			[ 'v_smallint', 0, false ],
			[ 'v_smallint', 32767, false ],		# MAX
			[ 'v_smallint', -32768, false ],	# MIN
			[ 'v_smallint', 32768, true ],		# 範囲外
			[ 'v_decimal', 0, false ],
			[ 'v_decimal', 9223372036854775808, false ],
			[ 'v_decimal', '9223372036854775808', false ],
			[ 'v_decimal', BigDecimal.new('9223372036854775808'), false ],
			[ 'v_real', 35.4343434343, false ],
			[ 'v_real', 35, false ],
			[ 'v_real', "35.4343434343", false ],
			[ 'v_real', "35", false ],
			[ 'v_real', BigDecimal.new('100.4343434343'), false ],
			[ 'v_double', 35.4343434343, false ],
			[ 'v_double', 35, false ],
			[ 'v_double', "35.4343434343", false ],
			[ 'v_double', "35", false ],
			[ 'v_double', BigDecimal.new('100.4343434343'), false ],
			[ 'v_text', "ABCDEFGHIJ", false ],
			[ 'v_text', 1234567890, false ],
			[ 'v_text', BigDecimal.new('100.4343434343'), false ],

			[ 'v_char_10', "ABCDEFGHIJ", false ],
			[ 'v_char_10', "ABCDEF", false ],
			[ 'v_char_10', "ABCDEFGHIJklmnopqrst", true ],
			[ 'v_char_10', 1234567890, false ],
			[ 'v_char_10', BigDecimal.new('100.4343434343'), true ],

			[ 'v_varchar_10', "ABCDEFGHIJ", false ],
			[ 'v_varchar_10', "ABCDEF", false ],
			[ 'v_varchar_10', "ABCDEFGHIJklmnopqrst", true ],
			[ 'v_varchar_10', 1234567890, false ],
			[ 'v_varchar_10', BigDecimal.new('100.4343434343'), true ],

			[ 'v_bytea', "\\x41424300444546", false ],
			[ 'v_bytea', 12345, false ],

			[ 'v_date', Date.new(2012, 10, 10), false ],
			[ 'v_date', '2012-10-10', false ],

			[ 'v_time', Time.new(1970, 1, 1, 13, 23, 10), false ],
			[ 'v_time', '13:23:10', false ],

			[ 'v_timestamp', '2012-10-10 13:23:10', false ],
			[ 'v_timestamp', DateTime.new(2012, 10, 10, 13, 23, 10), false ],
			[ 'v_timestamptz', '2012-10-10 13:23:10+0300', false ],
			[ 'v_timestamptz', DateTime.new(2012, 10, 10, 13, 23, 10, Rational(9, 24)), false ],

			[ 'v_bit_10', '0011000111', false ],
			[ 'v_varbit_10', '0011000111', false ],
			[ 'v_interval', '3 year 4 mon 5day', false ],
		];
		num_successed = 0;
		test_cases.each do |field, value, guess_excep|
			sql = "INSERT INTO various_types_tbl (#{field}) VALUES ($1);";
			if ! guess_excep
				@pq.execute(sql, value);
				num_successed += 1;
			else
				am_assert_exception(PgException, "#{field}") do
					@pq.execute(sql, value);
				end
			end

			sql = "SELECT count(*) AS ct FROM various_types_tbl;"
			am_assert_equals(num_successed, @pq.select_one(sql)['ct'].to_i,
				"various_types_tbl: #{field}, #{value}");

		end
	end

end

# ======================================================================
#
if __FILE__ == $0
	t = TestPgExec.new();
	t.test_pg_execute();
	t.test_pg_execute_with_type();
	t.test_pg_exec_prepared();
	t.test_pg_fetch();
	t.test_pg_execute_various_types();
end

