#! /usr/local/bin/ruby
# encoding: utf-8
#
# test_trigger.rb
#

require_relative 'am_assert';
include AmAssert;
require_relative 'pgx';

# ======================================================================
#
class TestTrigger
	# ==============================================================
	#
	def initialize()
		@pq = Pq.new('postgresql://localhost/db_plrb');
	end

	# ==============================================================
	#
	def test_trig_basic()
		@pq.execute("DELETE FROM trig_test_tbl;");

		# ======================================================
		# int_val == -3。trig_deny_minus_valの働きでINSERTされない。
		#
		@pq.execute("INSERT INTO trig_test_tbl (id, int_val) VALUES(1, -3);");
		result = @pq.select_one("SELECT COUNT(*) AS ct FROM trig_test_tbl;");
		am_assert_equals(0, result['ct']);

		# ======================================================
		# INSERT: int_val == 5。trig_deny_minus_valの働きで2倍になる。
		#
		@pq.execute("INSERT INTO trig_test_tbl (id, int_val) VALUES(2, 5);");
		result = @pq.select_one("SELECT int_val FROM trig_test_tbl WHERE id = 2;");
		am_assert_equals(10, result['int_val']);

		# ======================================================
		# UPDATE: int_val == 7。trig_deny_minus_valの働きで3倍になる。
		#
		@pq.execute("UPDATE trig_test_tbl SET int_val = 7 WHERE id = 2;");
		result = @pq.select_one("SELECT int_val FROM trig_test_tbl WHERE id = 2;");
		am_assert_equals(21, result['int_val']);

	end

	# ==============================================================
	# トリガーのカスケード
	# 「挿入」トリガー函数内で新たに「挿入」を実行することにより、
	# 次々に新たなレコードが挿入される。
	#
	def test_trig_cascade()
		@pq.execute("DELETE FROM trig_level_tbl;");
		@pq.execute("INSERT INTO trig_level_tbl (level, mark) VALUES (1, 'Mark-');");
		result = @pq.select_one("SELECT count(*) AS ct FROM trig_level_tbl;");
		am_assert_equals(15, result['ct']);
	end

end

# ======================================================================
#
if __FILE__ == $0
	t = TestTrigger.new();
	t.test_trig_basic();
	t.test_trig_cascade();
end

